/*


*/
#include <avr/io.h>
#include <avr/interrupt.h>
#include "t26Serial.h"


#define OUTPUT             1
#define INPUT              0


#define IsHigh(BIT, PORT)    ((PORT & (1<<BIT)) != 0)
#define IsLow(BIT, PORT)     ((PORT & (1<<BIT)) == 0)
#define SetBit(BIT, PORT)     PORT |= (1<<BIT)
#define ClearBit(BIT, PORT)   PORT &= ~(1<<BIT)

#define ABS(a)                ((a) < 0 ? -(a) : (a))
#define SIGN(x)               (x)==0?0:(x)>0?1:-1
#define NOP()                 asm volatile ("nop"::)


/*****************************| VARIABLES |********************************/
 
volatile unsigned int  AdcValues[10];


 
/************************| FUNCTION DECLARATIONS |*************************/
 
void AnalogInit (void);
int  Analog (int n);
void Delay(unsigned int delay);

/****************************| CODE... |***********************************/

int main (void) {
 
  int temp;
  unsigned char i;
  unsigned int lastSend[10];

  // set up directions 
    

  DDRA = (INPUT << PA0 | INPUT << PA1 | INPUT << PA2 | INPUT << PA3 | INPUT << PA4 | INPUT << PA5 | INPUT << PA6 | INPUT << PA7);
  DDRB = (INPUT << PB0 | OUTPUT << PB1 | INPUT << PB2 | INPUT << PB3 | INPUT << PB4 | INPUT << PB5 | INPUT << PB6 );

  AnalogInit();
  initSerial(103); // 9600 baud at 8Mhz
  
  // turn interrupts on
  sei();
  

  while(1){
  
    for (i = 0; i < 10; i++) {
     if(lastSend[i] != AdcValues[i]) {
       lastSend[i] = AdcValues[i];
       USART_printhex( i);
       sendChar('=');
       printHexWord(AdcValues[i]);
       sendChar(' ');
      }
    }
    
    sendChar('\r');
    sendChar('\n');  
    //Delay(65000);
  }

}

//------------------------| FUNCTIONS |------------------------

void Delay(unsigned int delay) {
  unsigned int x;
  for (x = delay; x != 0; x--) {
    NOP();
  }
}



void AnalogInit (void) {

  int i;

  // Activate ADC with Prescaler 
  ADCSR =   1 << ADEN  |
            1 << ADSC  | /* start a converstion, irq will take from here */
            0 << ADIF  |
            1 << ADIE  | /* enable interrupt */
            1 << ADPS2 | /* prescalar divide by 64 */
            1 << ADPS1 |
            0 << ADPS0 ;
                        
            
  for (i = 0; i < 10; AdcValues[i++] = 0);
  
}

int Analog (int n) {
  return AdcValues[n & 7];
}

ISR(ADC_vect) {
  unsigned char i;
  
  i = ADMUX & 31;                         // what channel we on?
  AdcValues[i] = ADC;                     // save value
  i = ((i + 1) % 10);                     // next value (0 thru 9, we will leave channel 10 cause its the reset line)
  ADMUX = i ;                             // select channel
  ADCSR |= _BV(ADSC);                     // start next conversion
  return;
}



























