/******************************************************************************
interrupt driven serial tx for tiny26

uses timer 1
    
*******************************************************************************/

#include"t26Serial.h"



// Bit positions

#define SetBit(BIT, PORT)        PORT |= (1<<BIT)
#define ClearBit(BIT, PORT)      PORT &= ~(1<<BIT)
#define NOP()                    asm volatile ("nop"::)

#define txd     1
#define serport PORTB

#define dataHigh()   SetBit(txd, serport)
#define dataLow()    ClearBit(txd, serport)

/*
int main (void)  {
 
  DDRA = (INPUT << PA0 | INPUT << PA1 | INPUT << PA2 | INPUT << PA3 | INPUT << PA4 | INPUT << PA5 |INPUT << PA6 |INPUT << PA7);      
  DDRB = (INPUT << PB0 | OUTPUT << PB1 | INPUT << PB2 | INPUT << PB3 | INPUT << PB4 | INPUT << PB5 |INPUT << PB6 |INPUT << PB7);
      
  initSerial(103); // 9600 baud at 8Mhz
  sei();   
                  
  while(1) {      
    sendChar('R'); while(status) ;
    sendChar('u'); while(status) ;
    sendChar('e'); while(status) ;
    sendChar(' '); while(status) ;
    sendChar('R'); while(status) ;
    sendChar('U'); while(status) ;
    sendChar('L'); while(status) ;
    sendChar('E'); while(status) ;     
    sendChar('S'); while(status) ;
    sendChar('\r'); while(status) ;
    sendChar('\n'); while(status) ;
  }
}*/
    
//------------------------| FUNCTIONS |------------------------

void inline printDigit(unsigned char n) {
  sendChar( (n & 0x0F) | 0x30 );
}

void printNumber8(unsigned char n) {
  unsigned char d;
  
  d = n/100;
  printDigit(d);
  n -= d * 100;
    
  d = n/10;
  printDigit(d);
  n -= d * 10;
  
  printDigit(n);
}

void printNumber16(unsigned int n) {
  unsigned int d;
  
  d = n/10000;
  if (d) printDigit(d);
  n -= d * 10000;
  
  d = n/1000;
  if (d) printDigit(d);
  n -= d * 1000;
  
  d = n/100;
  printDigit(d);
  n -= d * 100;
    
  d = n/10;
  printDigit(d);
  n -= d * 10;
  
  printDigit(n);
}


void printHexWord(unsigned int d) {
   USART_printhex(d >> 8);
   USART_printhex(d & 0xff);
}

void USART_printhex(uint8_t i) {
    uint8_t hi,lo;

    hi=i&0xF0;               // High nibble
    hi=hi>>4;
    hi=hi+'0';
    if (hi>'9')
        hi=hi+7;

    lo=(i&0x0F)+'0';         // Low nibble
    if (lo>'9')
        lo=lo+7;

  sendChar( hi );
  sendChar( lo );
}


void USART_printstring(char *data) {
    while(*data) {
        sendChar(*data);
        data++;
    }

}

void initSerial(unsigned char UBBR){
  // setup the io bit
  dataHigh();
  status = 0;
  initTimer(UBBR);
}

void initTimer(unsigned char top) {

  TCCR1B |= ((1<<CTC1) | (1<<CS12)); // clear on match, clk/8
  OCR1A  = top;                      // when to fire the interrupt
  OCR1C  = top;                      // when to rollover
  TIMSK |= (1<<OCIE1A);              // and do interrupt us
  
}



void sendChar(unsigned char data) {
 
  //if (status == 0) { // burn
    while(status != 0); // wait for it to be ready.
    
    txbuffer = data;
    status   = 10;
  
 // }      
}



ISR( TIMER1_CMPA_vect ) {
 TXService();
}

void TXService(void){
  static unsigned char mask;
  
  switch(status) {
    case 10: // send start bit
      mask = 0x01;
      dataLow();
      break;
    case 9:  
    case 8:
    case 7:
    case 6: 
    case 5:  
    case 4:
    case 3:
    case 2: // send bit
      if (mask & txbuffer) { dataHigh(); }
      else                 { dataLow(); }
      mask <<= 1;
      break;
      
    case 1: // send stop
      dataHigh();
      break;
      
    default:
      return;
  }
  status -= 1;
  return;
}


